@@ -44,6 +44,10 @@
 #include "util.h"
 #include "resize.h"
 #include "parse.h"
+
+/* djhjr - 4/19/96 */
+#include "gram.h"
+
 #include "list.h"
 #include "events.h"
 #include "menus.h"
@@ -56,6 +60,10 @@
 static char gray_bits[] = {
    0x02, 0x01};
 
+/* djhjr - 4/19/96 */
+static unsigned char black_bits[] = {
+   0xFF, 0xFF};
+
 int AddingX;
 int AddingY;
 int AddingW;
@@ -64,6 +72,7 @@ int AddingH;
 static int PlaceX = 50;
 static int PlaceY = 50;
 static void CreateWindowTitlebarButtons();
+void SetHighlightPixmap();
 
 char NoName[] = "Untitled"; /* name if no name is specified */
 
@@ -77,7 +86,7 @@ char NoName[] = "Untitled"; /* name if no name is specified */
  ************************************************************************
  */
 
-GetGravityOffsets (tmp, xp, yp)
+void GetGravityOffsets (tmp, xp, yp)
     TwmWindow *tmp;			/* window from which to get gravity */
     int *xp, *yp;			/* return values */
 {
@@ -108,8 +117,6 @@ GetGravityOffsets (tmp, xp, yp)
 }
 
 
-
-
 /***********************************************************************
  *
  *  Procedure:
@@ -137,7 +144,7 @@ IconMgr *iconp;
     XEvent event;
     unsigned long valuemask;		/* mask for create windows */
     XSetWindowAttributes attributes;	/* attributes for create windows */
-    int width, height;			/* tmp variable */
+    XTextProperty text_property;
     Atom actual_type;
     int actual_format;
     unsigned long nitems, bytesafter;
@@ -166,7 +173,17 @@ IconMgr *iconp;
 
     XSelectInput(dpy, tmp_win->w, PropertyChangeMask);
     XGetWindowAttributes(dpy, tmp_win->w, &tmp_win->attr);
-    XFetchName(dpy, tmp_win->w, &tmp_win->name);
+
+	/*
+	 * Ask the window manager for a name with "newer" R4 function -
+	 * it was 'XFetchName()', which apparently failed more often.
+	 * Submitted by Nicholas Jacobs
+	 */
+	if (XGetWMName(dpy, tmp_win->w, &text_property) != 0)
+		tmp_win->name = (char *)text_property.value;
+	else
+		tmp_win->name = NoName;
+
     tmp_win->class = NoClass;
     XGetClassHint(dpy, tmp_win->w, &tmp_win->class);
     FetchWmProtocols (tmp_win);
@@ -193,8 +210,6 @@ IconMgr *iconp;
 
     tmp_win->transient = Transient(tmp_win->w, &tmp_win->transientfor);
 
-    if (tmp_win->name == NULL)
-	tmp_win->name = NoName;
     if (tmp_win->class.res_name == NULL)
     	tmp_win->class.res_name = NoName;
     if (tmp_win->class.res_class == NULL)
@@ -278,6 +293,19 @@ IconMgr *iconp;
 
     tmp_win->old_bw = tmp_win->attr.border_width;
 
+	/* djhjr - 4/19/96 */
+    tmp_win->frame_bw3D = Scr->ThreeDBorderWidth;
+    if (LookInList(Scr->NoBorder, tmp_win->full_name, &tmp_win->class)) {
+	tmp_win->frame_bw = 0;
+	tmp_win->frame_bw3D = 0;
+    }
+    else
+    if (tmp_win->frame_bw3D != 0) {
+	tmp_win->frame_bw = 0;
+	Scr->ClientBorderWidth = FALSE;
+    }
+    else
+
     if (Scr->ClientBorderWidth) {
     	tmp_win->frame_bw = tmp_win->old_bw;
     } else {
@@ -416,26 +444,51 @@ IconMgr *iconp;
 		    break;
 	    }
 
+/* use initialized size... djhjr - 5/9/96
 	    width = (SIZE_HINDENT + XTextWidth (Scr->SizeFont.font,
 						tmp_win->name, namelen));
 	    height = Scr->SizeFont.height + SIZE_VINDENT * 2;
 
+* djhjr - 4/27/96
 	    XResizeWindow (dpy, Scr->SizeWindow, width + SIZE_HINDENT, height);
+*
+	    XResizeWindow (dpy, Scr->SizeWindow, Scr->SizeStringOffset +
+				Scr->SizeStringWidth, height);
+*/
+
 	    XMapRaised(dpy, Scr->SizeWindow);
 	    InstallRootColormap();
 
+/* DisplayPosition overwrites it anyway... djhjr - 5/9/96
 	    FBF(Scr->DefaultC.fore, Scr->DefaultC.back,
 		Scr->SizeFont.font->fid);
 	    XDrawImageString (dpy, Scr->SizeWindow, Scr->NormalGC,
 			      SIZE_HINDENT,
 			      SIZE_VINDENT + Scr->SizeFont.font->ascent,
 			      tmp_win->name, namelen);
+*/
 
+/* djhjr - 4/19/96
 	    AddingW = tmp_win->attr.width + bw2;
 	    AddingH = tmp_win->attr.height + tmp_win->title_height + bw2;
 
 		MoveOutline(Scr->Root, AddingX, AddingY, AddingW, AddingH,
 			    tmp_win->frame_bw, tmp_win->title_height);
+*/
+	    AddingW = tmp_win->attr.width + bw2 + 2 * tmp_win->frame_bw3D;
+	    AddingH = tmp_win->attr.height + tmp_win->title_height +
+				bw2 + 2 * tmp_win->frame_bw3D;
+		MoveOutline(Scr->Root, AddingX, AddingY, AddingW, AddingH,
+			    tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
+
+/* DisplayPosition() overwrites it anyway... djhjr - 5/9/96
+		* djhjr - 4/27/96 *
+	    XDrawImageString (dpy, Scr->SizeWindow, Scr->NormalGC, width,
+				SIZE_VINDENT + Scr->SizeFont.font->ascent, ": ", 2);
+*/
+
+		/* djhjr - 4/27/96 */
+	    DisplayPosition (tmp_win, AddingX, AddingY);
 
 	    while (TRUE)
 		{
@@ -500,14 +553,22 @@ IconMgr *iconp;
 			AddingY = Scr->MyDisplayHeight - AddingH;
 		}
 
+/* djhjr - 4/19/96
 		MoveOutline(Scr->Root, AddingX, AddingY, AddingW, AddingH,
 			    tmp_win->frame_bw, tmp_win->title_height);
+*/
+		MoveOutline(Scr->Root, AddingX, AddingY, AddingW, AddingH,
+			    tmp_win->frame_bw, tmp_win->title_height + tmp_win->frame_bw3D);
+
+		/* djhjr - 4/27/96 */
+		DisplayPosition (tmp_win, AddingX, AddingY);
 
 	    }
 
 	    if (event.xbutton.button == Button2) {
 		int lastx, lasty;
 
+/* AddStartResize() overwrites it anyway... djhjr - 5/9/96
 		Scr->SizeStringOffset = width +
 		  XTextWidth(Scr->SizeFont.font, ": ", 2);
 		XResizeWindow (dpy, Scr->SizeWindow, Scr->SizeStringOffset +
@@ -515,6 +576,8 @@ IconMgr *iconp;
 		XDrawImageString (dpy, Scr->SizeWindow, Scr->NormalGC, width,
 				  SIZE_VINDENT + Scr->SizeFont.font->ascent,
 				  ": ", 2);
+*/
+
 #if 0
 		if (0/*Scr->AutoRelativeResize*/)
 My R5 vtvwm came with this commented out, always 0.
@@ -585,7 +648,7 @@ Why?
 		    }
 
 		}
-	    }
+	    } /* if (event.xbutton.button == Button2) */
 	    else if (event.xbutton.button == Button3)
 	    {
 		int maxw = Scr->MyDisplayWidth - AddingX - bw2;
@@ -615,8 +678,14 @@ Why?
 
 	    tmp_win->attr.x = AddingX;
 	    tmp_win->attr.y = AddingY + tmp_win->title_height;
+
+/* djhjr - 4/19/96
 	    tmp_win->attr.width = AddingW - bw2;
 	    tmp_win->attr.height = AddingH - tmp_win->title_height - bw2;
+*/
+	    tmp_win->attr.width = AddingW - bw2 - 2 * tmp_win->frame_bw3D;
+	    tmp_win->attr.height = AddingH - tmp_win->title_height -
+				bw2 - 2 * tmp_win->frame_bw3D;
 
 	    XUngrabServer(dpy);
 	}
@@ -626,9 +695,13 @@ Why?
 #if 0
 	    if (Scr->GeometriesAreVirtual)
 #endif
-		if (Scr->GeometriesAreVirtual || ( !Scr->GeometriesAreVirtual &&
-		      (tmp_win->nailed||(Scr->FixTransientVirtualGeometries
-		                         &&tmp_win->transient))))
+/* added 'FixManagedVirtualGeometries' - djhjr - 1/6/98 */
+		if (Scr->GeometriesAreVirtual ||
+			(!Scr->GeometriesAreVirtual &&
+				(tmp_win->nailed ||
+				(Scr->FixManagedVirtualGeometries && !tmp_win->transient ||
+				(Scr->FixTransientVirtualGeometries && tmp_win->transient))
+		)))
 
 		/* if NotVirtualGeometries is set, should also do this if it's
 		   a sticky or (it's a transient window and
@@ -653,7 +726,12 @@ Why?
 #endif
 
     if (!Scr->ClientBorderWidth) {	/* need to adjust for twm borders */
+
+/* djhjr - 4/19/96
 	int delta = tmp_win->attr.border_width - tmp_win->frame_bw;
+*/
+	int delta = tmp_win->attr.border_width - tmp_win->frame_bw - tmp_win->frame_bw3D;
+
 	tmp_win->attr.x += gravx * delta;
 	tmp_win->attr.y += gravy * delta;
     }
@@ -703,7 +781,11 @@ Why?
 
     /* get all the colors for the window */
 
+/* djhjr - 4/25/96
     tmp_win->border = Scr->BorderColor;
+*/
+	tmp_win->border.back = Scr->BorderColor;
+
     tmp_win->icon_border = Scr->IconBorderColor;
     tmp_win->border_tile.fore = Scr->BorderTileC.fore;
     tmp_win->border_tile.back = Scr->BorderTileC.back;
@@ -714,8 +796,13 @@ Why?
     tmp_win->virtual.fore = Scr->VirtualDesktopDisplayC.fore;
     tmp_win->virtual.back = Scr->VirtualDesktopDisplayC.back;
 
+/* djhjr - 4/25/96
     GetColorFromList(Scr->BorderColorL, tmp_win->full_name, &tmp_win->class,
 	&tmp_win->border);
+*/
+    GetColorFromList(Scr->BorderColorL, tmp_win->full_name, &tmp_win->class,
+	&tmp_win->border.back);
+
     GetColorFromList(Scr->IconBorderColorL, tmp_win->full_name, &tmp_win->class,
 	&tmp_win->icon_border);
     GetColorFromList(Scr->BorderTileForegroundL, tmp_win->full_name,
@@ -739,17 +826,35 @@ Why?
 	    GetColorFromList(Scr->TitleForegroundL, tmp_win->full_name,
 			     &tmp_win->class, &tmp_win->virtual.back);
 
+	/* djhjr - 4/19/96 */
+    if (Scr->use3Dtitles  && !Scr->BeNiceToColormap) GetShadeColors (&tmp_win->title);
+    if (Scr->ButtonColorIsFrame || (Scr->use3Dborders && !Scr->BeNiceToColormap)) {
+	GetShadeColors (&tmp_win->border);
+	GetShadeColors (&tmp_win->border_tile);
+    }
 
     /* create windows */
 
+/* djhjr - 4/19/96
     tmp_win->frame_x = tmp_win->attr.x + tmp_win->old_bw - tmp_win->frame_bw;
     tmp_win->frame_y = tmp_win->attr.y - tmp_win->title_height +
 	tmp_win->old_bw - tmp_win->frame_bw;
     tmp_win->frame_width = tmp_win->attr.width;
     tmp_win->frame_height = tmp_win->attr.height + tmp_win->title_height;
+*/
+    tmp_win->frame_x = tmp_win->attr.x + tmp_win->old_bw - tmp_win->frame_bw
+			- tmp_win->frame_bw3D;
+    tmp_win->frame_y = tmp_win->attr.y - tmp_win->title_height +
+	tmp_win->old_bw - tmp_win->frame_bw - tmp_win->frame_bw3D;
+    tmp_win->frame_width = tmp_win->attr.width + 2 * tmp_win->frame_bw3D;
+    tmp_win->frame_height = tmp_win->attr.height + tmp_win->title_height +
+				2 * tmp_win->frame_bw3D;
+    ConstrainSize (tmp_win, &tmp_win->frame_width, &tmp_win->frame_height);
+
     tmp_win->virtual_frame_x = R_TO_V_X(tmp_win->frame_x);
     tmp_win->virtual_frame_y = R_TO_V_Y(tmp_win->frame_y);
 
+/* djhjr - 4/19/96
     valuemask = CWBackPixmap | CWBorderPixel | CWCursor | CWEventMask;
     attributes.background_pixmap = None;
     attributes.border_pixel = tmp_win->border;
@@ -761,6 +866,36 @@ Why?
 	attributes.save_under = True;
 	valuemask |= CWSaveUnder;
     }
+*/
+/* djhjr - 9/14/96
+    valuemask = CWBackPixmap | CWBorderPixel | CWCursor | CWEventMask | CWBackPixel;
+    attributes.background_pixmap = None;
+*/
+    valuemask = CWBorderPixel | CWCursor | CWEventMask | CWBackPixel;
+
+    attributes.background_pixel	 = tmp_win->border.back;
+    attributes.border_pixel = tmp_win->border.back;
+    attributes.cursor = Scr->FrameCursor;
+    attributes.event_mask = (SubstructureRedirectMask | 
+			     ButtonPressMask | ButtonReleaseMask |
+			     EnterWindowMask | LeaveWindowMask | ExposureMask);
+    if (tmp_win->attr.save_under) {
+	attributes.save_under = True;
+	valuemask |= CWSaveUnder;
+    }
+
+	/* djhjr - 9/17/96 - slows down iconify/delete/destroy too much...
+	if (Scr->BackingStore)
+	{
+		attributes.backing_store = WhenMapped;
+		valuemask |= CWBackingStore;
+	}
+	*/
+
+    if (tmp_win->hints.flags & PWinGravity) {
+	attributes.win_gravity = tmp_win->hints.win_gravity;
+	valuemask |= CWWinGravity;
+    }
 
     tmp_win->frame = XCreateWindow (dpy, Scr->Root, tmp_win->frame_x,
 				    tmp_win->frame_y,
@@ -776,11 +911,27 @@ Why?
 	valuemask = (CWEventMask | CWBorderPixel | CWBackPixel);
 	attributes.event_mask = (KeyPressMask | ButtonPressMask |
 				 ButtonReleaseMask | ExposureMask);
+/* djhjr - 4/19/96
 	attributes.border_pixel = tmp_win->border;
+*/
+	attributes.border_pixel = tmp_win->title.back;
 	attributes.background_pixel = tmp_win->title.back;
+
+	/* djhjr - 9/17/96 */
+	if (Scr->BackingStore)
+	{
+		attributes.backing_store = WhenMapped;
+		valuemask |= CWBackingStore;
+	}
+
 	tmp_win->title_w = XCreateWindow (dpy, tmp_win->frame,
+/* djhjr - 4/19/96
 					  -tmp_win->frame_bw,
 					  -tmp_win->frame_bw,
+*/
+					  tmp_win->frame_bw3D - tmp_win->frame_bw,
+					  tmp_win->frame_bw3D - tmp_win->frame_bw,
+
 					  (unsigned int) tmp_win->attr.width,
 					  (unsigned int) Scr->TitleHeight,
 					  (unsigned int) tmp_win->frame_bw,
@@ -796,6 +947,15 @@ Why?
 
     if (tmp_win->highlight)
     {
+
+	/* djhjr - 4/19/96 */
+	if (Scr->use3Dtitles && (Scr->Monochrome != COLOR))
+	    tmp_win->gray = XCreatePixmapFromBitmapData(dpy, Scr->Root, 
+		(char *)black_bits, gray_width, gray_height, 
+		tmp_win->border_tile.fore, tmp_win->border_tile.back,
+		Scr->d_depth);
+	else
+
 	tmp_win->gray = XCreatePixmapFromBitmapData(dpy, Scr->Root,
 	    gray_bits, gray_width, gray_height,
 	    tmp_win->border_tile.fore, tmp_win->border_tile.back,
@@ -815,6 +975,12 @@ Why?
 	XDefineCursor(dpy, tmp_win->title_w, Scr->TitleCursor);
     }
 
+	/* djhjr - 4/19/96 */
+    else {
+	tmp_win->title_x = tmp_win->frame_bw3D - tmp_win->frame_bw;
+	tmp_win->title_y = tmp_win->frame_bw3D - tmp_win->frame_bw;
+    }
+
     valuemask = (CWEventMask | CWDontPropagate);
     attributes.event_mask = (StructureNotifyMask | PropertyChangeMask |
 			     ColormapChangeMask | VisibilityChangeMask |
@@ -844,7 +1010,12 @@ Why?
     if (!tmp_win->iconmgr)
 	XAddToSaveSet(dpy, tmp_win->w);
 
+/* djhjr - 4/19/96
     XReparentWindow(dpy, tmp_win->w, tmp_win->frame, 0, tmp_win->title_height);
+*/
+    XReparentWindow(dpy, tmp_win->w, tmp_win->frame, tmp_win->frame_bw3D,
+		tmp_win->title_height + tmp_win->frame_bw3D);
+
     /*
      * Reparenting generates an UnmapNotify event, followed by a MapNotify.
      * Set the map state to FALSE to prevent a transition back to
@@ -955,7 +1126,7 @@ static void do_add_binding (button, context, modifier, func)
     mb->item = NULL;
 }
 
-AddDefaultBindings ()
+void AddDefaultBindings ()
 {
     /*
      * The bindings are stored in Scr->Mouse, indexed by
@@ -1110,9 +1281,15 @@ static Window CreateHighlightWindow (tmp_win)
      * solid line.
      */
     if (!Scr->hilitePm) {
-	Scr->hilitePm = XCreateBitmapFromData (dpy, tmp_win->title_w,
-					       gray_bits, gray_width,
-					       gray_height);
+
+	/* djhjr - 4/20/96 */
+	if (Scr->use3Dtitles && (Scr->Monochrome != COLOR))
+	    Scr->hilitePm = XCreateBitmapFromData (dpy, tmp_win->title_w, 
+					(char *)black_bits, gray_width, gray_height);
+	else
+	    Scr->hilitePm = XCreateBitmapFromData (dpy, tmp_win->title_w, 
+					gray_bits, gray_width, gray_height);
+
 	Scr->hilite_pm_width = gray_width;
 	Scr->hilite_pm_height = gray_height;
     }
@@ -1144,6 +1321,20 @@ static Window CreateHighlightWindow (tmp_win)
 	attributes.background_pixel = tmp_win->title.fore;
     }
 
+	/* djhjr - 4/19/96 */
+    if (Scr->use3Dtitles)
+/* djhjr - 4/25/96
+	w = XCreateWindow (dpy, tmp_win->title_w, 0, Scr->FramePadding + 2,
+		       (unsigned int) Scr->TBInfo.width, (unsigned int) (h - 4),
+*/
+	w = XCreateWindow (dpy, tmp_win->title_w, 0, Scr->FramePadding + 3,
+		       (unsigned int) Scr->TBInfo.width, (unsigned int) (h - 6),
+
+		       (unsigned int) 0,
+		       Scr->d_depth, (unsigned int) CopyFromParent,
+		       Scr->d_visual, valuemask, &attributes);
+    else
+
     w = XCreateWindow (dpy, tmp_win->title_w, 0, Scr->FramePadding,
 		       (unsigned int) Scr->TBInfo.width, (unsigned int) h,
 		       (unsigned int) 0,
@@ -1173,77 +1364,89 @@ void ComputeCommonTitleOffsets ()
 
 void ComputeWindowTitleOffsets (tmp_win, width, squeeze)
     TwmWindow *tmp_win;
+	int width;
     Bool squeeze;
 {
     tmp_win->highlightx = (Scr->TBInfo.titlex + tmp_win->name_width);
+
+	/* djhjr - 4/19/96
+    if (Scr->use3Dtitles) tmp_win->highlightx += 6;
+	*/
+
     if (tmp_win->hilite_w || Scr->TBInfo.nright > 0)
       tmp_win->highlightx += Scr->TitlePadding;
+
     tmp_win->rightx = width - Scr->TBInfo.rightoff;
-    if (squeeze && tmp_win->squeeze_info) {
-	int rx = (tmp_win->highlightx +
-		  (tmp_win->hilite_w
-		    ? Scr->TBInfo.width * 2 : 0) +
-		  (Scr->TBInfo.nright > 0 ? Scr->TitlePadding : 0) +
-		  Scr->FramePadding);
-	if (rx < tmp_win->rightx) tmp_win->rightx = rx;
+
+	if (squeeze && tmp_win->squeeze_info)
+	{
+		int rx;
+
+		/* djhjr - 3/13/97 */
+		if (Scr->use3Dtitles)
+		{
+			rx = (tmp_win->highlightx + Scr->TBInfo.width * 2 +
+				(Scr->TBInfo.nright > 0 ? Scr->TitlePadding : 0) +
+				Scr->FramePadding);
+		}
+		else
+		{
+			rx = (tmp_win->highlightx +
+				(tmp_win->hilite_w ? Scr->TBInfo.width * 2 : 0) +
+				(Scr->TBInfo.nright > 0 ? Scr->TitlePadding : 0) +
+				Scr->FramePadding);
+		}
+
+		if (rx < tmp_win->rightx) tmp_win->rightx = rx;
     }
+
     return;
 }
 
 
 /*
- * ComputeTitleLocation - calculate the position of the title window; we need
- * to take the frame_bw into account since we want (0,0) of the title window
- * to line up with (0,0) of the frame window.
+ * ComputeTitleLocation - calculate the position of the title window.
+ *
+ * substantially re-written - djhjr - 1/8/98
  */
 void ComputeTitleLocation (tmp)
-    register TwmWindow *tmp;
+    TwmWindow *tmp;
 {
-    tmp->title_x = -tmp->frame_bw;
-    tmp->title_y = -tmp->frame_bw;
+	tmp->title_x = tmp->frame_bw3D - tmp->frame_bw;
+	tmp->title_y = tmp->frame_bw3D - tmp->frame_bw;
 
-    if (tmp->squeeze_info) {
-	register SqueezeInfo *si = tmp->squeeze_info;
-	int basex;
-	int maxwidth = tmp->frame_width;
-	int tw = tmp->title_width;
+	if (tmp->squeeze_info)
+	{
+		SqueezeInfo *si = tmp->squeeze_info;
+		int basex;
+		int fw = tmp->frame_bw + tmp->frame_bw3D;
 
-	/*
-	 * figure label base from squeeze info (justification fraction)
-	 */
-	if (si->denom == 0) {	/* num is pixel based */
-	    if ((basex = si->num) == 0) {  /* look for special cases */
-		switch (si->justify) {
-		  case J_RIGHT:
-		    basex = maxwidth;
-		    break;
-		  case J_CENTER:
-		    basex = maxwidth / 2;
-		break;
-		}
-	    }
-	} else {			/* num/denom is fraction */
-	    basex = ((si->num * maxwidth) / si->denom);
-	    if (si->num < 0) basex += maxwidth;
+		if (si->denom != 0 && si->num != 0)
+			basex = ((tmp->frame_width - tmp->title_width) / si->denom) * si->num + fw;
+		else
+			if (si->denom == 0 && si->num != 0)
+				basex = si->num + fw;
+			else
+				switch (si->justify)
+				{
+					case J_LEFT:
+						basex = tmp->title_x + fw;
+						break;
+					case J_CENTER:
+						basex = tmp->frame_width / 2 - (tmp->title_width / 2 - fw);
+						break;
+					case J_RIGHT:
+						basex = tmp->frame_width - tmp->title_width;
+						break;
+				}
+
+		if (basex > tmp->frame_width - tmp->title_width)
+			basex = tmp->frame_width - tmp->title_width;
+		if (basex < 0)
+			basex = tmp->title_x + fw;
+
+		tmp->title_x = basex - fw;
 	}
-
-	/*
-	 * adjust for left (nop), center, right justify and clip
-	 */
-	switch (si->justify) {
-	  case J_CENTER:
-	    basex -= tw / 2;
-	    break;
-	  case J_RIGHT:
-	    basex -= tw - 1;
-	    break;
-	}
-	if (basex > maxwidth - tw + 1)
-	  basex = maxwidth - tw + 1;
-	if (basex < 0) basex = 0;
-
-	tmp->title_x = basex - tmp->frame_bw;
-    }
 }
 
 
@@ -1272,14 +1475,13 @@ static void CreateWindowTitlebarButtons (tmp_win)
     leftx = y = Scr->TBInfo.leftx;
     rightx = tmp_win->rightx;
 
-    attributes.win_gravity = NorthWestGravity;
     attributes.background_pixel = tmp_win->title.back;
     attributes.border_pixel = tmp_win->title.fore;
     attributes.event_mask = (ButtonPressMask | ButtonReleaseMask |
 			     ExposureMask);
     attributes.cursor = Scr->ButtonCursor;
-    valuemask = (CWWinGravity | CWBackPixel | CWBorderPixel | CWEventMask |
-		 CWCursor);
+
+    valuemask = (CWWinGravity | CWBackPixel | CWBorderPixel | CWEventMask | CWCursor);
 
     tmp_win->titlebuttons = NULL;
     nb = Scr->TBInfo.nleft + Scr->TBInfo.nright;
@@ -1305,13 +1507,45 @@ static void CreateWindowTitlebarButtons (tmp_win)
 		    leftx += boxwidth;
 		    attributes.win_gravity = NorthWestGravity;
 		}
+
 		tbw->window = XCreateWindow (dpy, tmp_win->title_w, x, y, h, h,
 					     (unsigned int) Scr->TBInfo.border,
 					     0, (unsigned int) CopyFromParent,
 					     (Visual *) CopyFromParent,
 					     valuemask, &attributes);
+
+		/* djhjr - 9/14/96 - moved from InitTitlebarButtons()... */
+		/* djhjr - 9/21/96 */
+		if (!(tb->image = GetImage (tb->name,
+			(Scr->ButtonColorIsFrame) ? tmp_win->border : tmp_win->title)))
+		    if (!(tb->image = GetImage (TBPM_QUESTION,
+				(Scr->ButtonColorIsFrame) ? tmp_win->border : tmp_win->title)))
+				fprintf (stderr, "%s:  unable to add titlebar button \"%s\"\n",
+					ProgramName, tb->name);
+
+		tb->width  = tb->image->width;
+		tb->height = tb->image->height;
+
+		tb->dstx = (h - tb->width + 1) / 2;
+		if (tb->dstx < 0) {		/* clip to minimize copying */
+	    	tb->srcx = -(tb->dstx);
+		    tb->width = h;
+		    tb->dstx = 0;
+		} else {
+		    tb->srcx = 0;
+		}
+		tb->dsty = (h - tb->height + 1) / 2;
+		if (tb->dsty < 0) {
+	    	tb->srcy = -(tb->dsty);
+		    tb->height = h;
+		    tb->dsty = 0;
+		} else {
+		    tb->srcy = 0;
+		}
+		/* ...end of moved */
+
 		tbw->info = tb;
-	    }
+	    } /* end for(...) */
 	}
     }
 
@@ -1319,13 +1553,18 @@ static void CreateWindowTitlebarButtons (tmp_win)
 			 ? CreateHighlightWindow (tmp_win) : None);
 
     XMapSubwindows(dpy, tmp_win->title_w);
+
+/* djhjr - 4/25/96
     if (tmp_win->hilite_w)
       XUnmapWindow(dpy, tmp_win->hilite_w);
+*/
+	PaintTitleHighlight(tmp_win, off);
+
     return;
 }
 
 
-SetHighlightPixmap (filename)
+void SetHighlightPixmap (filename)
     char *filename;
 {
     Pixmap pm = GetBitmap (filename);
@@ -1341,7 +1580,7 @@ SetHighlightPixmap (filename)
 }
 
 
-FetchWmProtocols (tmp)
+void FetchWmProtocols (tmp)
     TwmWindow *tmp;
 {
     unsigned long flags = 0L;
@@ -1438,7 +1677,7 @@ CreateColormapWindow(w, creating_parent, property_window)
     return (cwin);
 }
 
-FetchWmColormapWindows (tmp)
+void FetchWmColormapWindows (tmp)
     TwmWindow *tmp;
 {
     register int i, j;
@@ -1586,9 +1825,19 @@ void GetWindowSizeHints (tmp)
 			       NorthEastGravity, NorthWestGravity };
 	int right =  tmp->attr.x + tmp->attr.width + 2 * tmp->old_bw;
 	int bottom = tmp->attr.y + tmp->attr.height + 2 * tmp->old_bw;
+
+/* djhjr - 4/19/96
 	tmp->hints.win_gravity =
 	  gravs[((Scr->MyDisplayHeight - bottom < tmp->title_height) ? 0 : 2) |
 		((Scr->MyDisplayWidth - right   < tmp->title_height) ? 0 : 1)];
+*/
+	tmp->hints.win_gravity = 
+	  gravs[((Scr->MyDisplayHeight - bottom <
+		tmp->title_height + 2 * tmp->frame_bw3D) ? 0 : 2) |
+		((Scr->MyDisplayWidth - right   <
+		tmp->title_height + 2 * tmp->frame_bw3D) ? 0 : 1)];
+
 	tmp->hints.flags |= PWinGravity;
     }
 }
+
