@@ -47,9 +47,8 @@
 #include "parse.h"
 #include <X11/Xatom.h>
 
-#ifndef SYSTEM_INIT_FILE
-#define SYSTEM_INIT_FILE "/usr/lib/X11/twm/system.twmrc"
-#endif
+extern void twmrc_error_prefix();
+
 #define BUF_LEN 300
 
 static FILE *twmrc;
@@ -60,6 +59,8 @@ static char overflowbuff[20];		/* really only need one */
 static int overflowlen;
 static char **stringListSource, *currentString;
 static int ParseUsePPosition();
+void put_pixel_on_root();
+
 int RaiseDelay = 0;           /* msec, for AutoRaise *//*RAISEDELAY*/
 extern int yylineno;
 extern int mods;
@@ -120,10 +121,19 @@ int ParseTwmrc (filename)
 
     /*
      * If filename given, try it, else try ~/.vtwmrc.#, else try ~/.vtwmrc,
-     * else try ~/.twmrc.#, else ~/.twmrc, else system.twmrc; finally using
-     * built-in defaults.
+     * else try system.vtwmrc, else try ~/.twmrc.#, else ~/.twmrc, else
+     * system.twmrc; finally using built-in defaults.
+     *
+     * This choice allows user, then system, versions of .vtwmrc, followed
+     * by user, then system, versions of .twmrc.
+     * Thus, sites that have both twm and vtwm can allow users without
+     * private .vtwmrc or .twmrc files to fall back to system-wide
+     * defaults (very important when there are many users), yet the
+     * presence of a private .twmrc file for twm will not prevent
+     * features of a system-wide .vtwmrc file from exploiting the mew
+     * features of vtwm. Submitted by Nelson H. F. Beebe
      */
-    for (twmrc = NULL, i = 0; !twmrc && i < 6; i++) {
+    for (twmrc = NULL, i = 0; !twmrc && i <= 6; i++) {
 	switch (i) {
 	  case 0:			/* -f filename */
 	    cp = filename;
@@ -148,7 +158,11 @@ int ParseTwmrc (filename)
 	    }
 	    break;
 
-	  case 3:			/* ~/.twmrc.screennum */
+ 	  case 3:			/* system.vtwmrc */
+ 	    cp = SYSTEM_VTWMRC;
+ 	    break;
+ 
+ 	  case 4:			/* ~/.twmrc.screennum */
 	    if (!filename) {
 		home = getenv ("HOME");
 		if (home) {
@@ -161,14 +175,14 @@ int ParseTwmrc (filename)
 	    }
 	    continue;
 
-	  case 4:			/* ~/.twmrc */
+	  case 5:			/* ~/.twmrc */
 	    if (home) {
 		tmpfilename[homelen + 7] = '\0';
 	    }
 	    break;
 
-	  case 5:			/* system.twmrc */
-	    cp = SYSTEM_INIT_FILE;
+	  case 6:			/* system.twmrc */
+	    cp = SYSTEM_TWMRC;
 	    break;
 	}
 
@@ -356,6 +370,27 @@ typedef struct _TwmKeyword {
 
 #define kw0_WarpSnug                         46 /* DSE */
 
+/* djhjr - 4/18/96 */
+#define kw0_Use3DMenus			47
+#define kw0_Use3DTitles			48
+#define kw0_Use3DIconManagers	49
+#define kw0_Use3DBorders		50
+
+/* djhjr - 4/25/96 */
+#define kw0_SunkFocusWindowTitle			51
+
+/* djhjr - 6/25/96 */
+#define kw0_ShallowReliefWindowButton		52
+
+/* djhjr - 9/21/96 */
+#define kw0_ButtonColorIsFrame				53
+
+/* djhjr - 1/6/98 */
+#define kw0_FixManagedVirtualGeometries		54
+
+/* djhjr - 1/19/98 */
+#define kw0_BeNiceToColormap				55
+
 #define kws_UsePPosition		1
 #define kws_IconFont			2
 #define kws_ResizeFont			3
@@ -369,6 +404,12 @@ typedef struct _TwmKeyword {
 #define kws_DoorFont			11
 #define kws_MenuTitleFont       12 /* DSE */
 
+/* djhjr - 5/10/96 */
+#define kws_InfoFont			13
+
+/* djhjr - 5/15/96 */
+#define kws_ResizeRegion		14
+
 #define kwn_ConstrainedMoveTime     1
 #define kwn_MoveDelta               2
 #define kwn_XorValue                3
@@ -387,6 +428,11 @@ typedef struct _TwmKeyword {
 #define kwn_RealScreenBorderWidth  16 /* DSE */
 #define kwn_AutoPanWarpWithRespectToRealScreen 17 /* DSE */
 
+/* djhjr - 4/18/96 */
+#define kwn_ThreeDBorderWidth		18
+#define kwn_ClearShadowContrast		19
+#define kwn_DarkShadowContrast		20
+
 #define kwcl_BorderColor		1
 #define kwcl_IconManagerHighlight	2
 #define kwcl_BorderTileForeground	3
@@ -432,20 +478,36 @@ static TwmKeyword keytable[] = {
     { "autoraise",		AUTO_RAISE, 0 },
 	{ "autoraisedelay",       NKEYWORD, kwn_RaiseDelay },/*RAISEDELAY*/
     { "autorelativeresize",	KEYWORD, kw0_AutoRelativeResize },
+
+	/* djhjr - 1/19/98 */
+    { "benicetocolormap",	KEYWORD, kw0_BeNiceToColormap },
+
     { "bordercolor",		CLKEYWORD, kwcl_BorderColor },
     { "bordertilebackground",	CLKEYWORD, kwcl_BorderTileBackground },
     { "bordertileforeground",	CLKEYWORD, kwcl_BorderTileForeground },
     { "borderwidth",		NKEYWORD, kwn_BorderWidth },
     { "button",			BUTTON, 0 },
+
+	/* djhjr - 9/21/96 */
+    { "buttoncolorisframe",	KEYWORD, kw0_ButtonColorIsFrame },
+
     { "buttonindent",		NKEYWORD, kwn_ButtonIndent },
     { "c",			CONTROL, 0 },
     { "center",			JKEYWORD, J_CENTER },
+
+    /* djhjr - 4/19/96 */
+    { "clearshadowcontrast",	NKEYWORD, kwn_ClearShadowContrast },
+
     { "clientborderwidth",	KEYWORD, kw0_ClientBorderWidth },
     { "color",			COLOR, 0 },
     { "constrainedmovetime",	NKEYWORD, kwn_ConstrainedMoveTime },
     { "control",		CONTROL, 0 },
     { "cursors",		CURSORS, 0 },
     { "d",			VIRTUAL_WIN, 0 },
+
+    /* djhjr - 4/19/96 */
+    { "darkshadowcontrast",	NKEYWORD, kwn_DarkShadowContrast },
+
     { "decoratetransients",	KEYWORD, kw0_DecorateTransients },
     { "defaultbackground",	CKEYWORD, kwc_DefaultBackground },
     { "defaultforeground",	CKEYWORD, kwc_DefaultForeground },
@@ -531,6 +593,10 @@ static TwmKeyword keytable[] = {
     { "f.rightzoom",		FKEYWORD, F_RIGHTZOOM },
     { "f.ring",		FKEYWORD, F_RING },
     { "f.saveyourself",		FKEYWORD, F_SAVEYOURSELF },
+
+	/* djhjr - 4/30/96 */
+    { "f.separator",		FKEYWORD, F_SEPARATOR },
+
     { "f.setrealscreen",	FSKEYWORD, F_SETREALSCREEN },
     { "f.showdesktopdisplay",	FKEYWORD, F_SHOWDESKTOP },
     { "f.showiconmgr",		FKEYWORD, F_SHOWLIST },
@@ -565,6 +631,10 @@ static TwmKeyword keytable[] = {
     { "f.winrefresh",		FKEYWORD, F_WINREFRESH },
     { "f.zoom",			FKEYWORD, F_ZOOM },
     { "f.zoomzoom",			FKEYWORD, F_ZOOMZOOM },
+
+    /* djhjr - 1/6/98 */
+    { "fixmanagedvirtualgeometries", KEYWORD, kw0_FixManagedVirtualGeometries },
+
     { "fixtransientvirtualgeometries", KEYWORD, 
         kw0_FixTransientVirtualGeometries },                          /* DSE */
     { "forceicons",		KEYWORD, kw0_ForceIcons },
@@ -591,6 +661,10 @@ static TwmKeyword keytable[] = {
     { "iconmgr",		ICONMGR, 0 },
     { "iconregion",		ICON_REGION, 0 },
     { "icons",			ICONS, 0 },
+
+	/* djhjr - 5/10/96 */
+    { "infofont",		SKEYWORD, kws_InfoFont },
+
     { "interpolatemenucolors",	KEYWORD, kw0_InterpolateMenuColors },
     { "l",			LOCK, 0 },
     { "left",			JKEYWORD, J_LEFT },
@@ -626,6 +700,10 @@ static TwmKeyword keytable[] = {
     { "nograbserver",		KEYWORD, kw0_NoGrabServer },
     { "nohighlight",		NO_HILITE, 0 },
     { "noiconifyiconmanagers",	KEYWORD, kw0_NoIconifyIconManagers }, /* PF */
+
+    /* djhjr - 1/27/98 */
+    { "noiconmanagerhighlight",		NO_ICONMGR_HILITE, 0 },
+
     { "noiconmanagers",		KEYWORD, kw0_NoIconManagers },
     { "nomenushadows",		KEYWORD, kw0_NoMenuShadows },
     { "noraiseondeiconify",	KEYWORD, kw0_NoRaiseOnDeiconify },
@@ -656,6 +734,10 @@ static TwmKeyword keytable[] = {
     { "realscreenpixmap",		REALSCREENMAP, 0 },/*RFB PIXMAP*/
     { "resize",			RESIZE, 0 },
     { "resizefont",		SKEYWORD, kws_ResizeFont },
+
+	/* djhjr - 5/15/96 */
+    { "resizeregion",		SKEYWORD, kws_ResizeRegion },
+
     { "restartpreviousstate",	KEYWORD, kw0_RestartPreviousState },
     { "rhspulldownmenus", KEYWORD, kw0_RightHandSidePulldownMenus }, /* DSE */
     { "right",			JKEYWORD, J_RIGHT },
@@ -665,6 +747,10 @@ static TwmKeyword keytable[] = {
     { "s",			SHIFT, 0 },
     { "savecolor",              SAVECOLOR, 0},
     { "select",			SELECT, 0 },
+
+	/* djhjr - 6/25/96 */
+    { "shallowreliefwindowbutton",	KEYWORD, kw0_ShallowReliefWindowButton },
+
     { "shift",			SHIFT, 0 },
     { "showiconmanager",	KEYWORD, kw0_ShowIconManager },
     { "snaprealscreen",		KEYWORD, kw0_SnapRealScreen },
@@ -676,7 +762,15 @@ static TwmKeyword keytable[] = {
     { "stayupoptionalmenus",	KEYWORD, kw0_StayUpOptionalMenus }, /* PF */
     { "sticky",             NAILEDDOWN, 0 },/*RFB*/
     { "stickyabove",        KEYWORD, kw0_StickyAbove },                /* DSE */
+
+	/* djhjr - 4/25/96 */
+    { "sunkfocuswindowtitle",	KEYWORD, kw0_SunkFocusWindowTitle },
+
     { "t",			TITLE, 0 },
+
+    /* djhjr - 4/18/96 */
+    { "threedborderwidth",	NKEYWORD, kwn_ThreeDBorderWidth },
+
     { "title",			TITLE, 0 },
     { "titlebackground",	CLKEYWORD, kwcl_TitleBackground },
     { "titlebuttonborderwidth",	NKEYWORD, kwn_TitleButtonBorderWidth },
@@ -687,6 +781,13 @@ static TwmKeyword keytable[] = {
     { "unknownicon",		SKEYWORD, kws_UnknownIcon },
     { "usepposition",		SKEYWORD, kws_UsePPosition },
 	{ "userealscreenborder", KEYWORD, kw0_UseRealScreenBorder },/*RFB*/
+
+	/* djhjr - 4/18/96 */
+    { "usethreedborders",	KEYWORD, kw0_Use3DBorders },
+    { "usethreediconmanagers",	KEYWORD, kw0_Use3DIconManagers },
+    { "usethreedmenus",		KEYWORD, kw0_Use3DMenus },
+    { "usethreedtitles",	KEYWORD, kw0_Use3DTitles },
+
     { "v",			VIRTUAL, 0 },
     { "virtual",		VIRTUAL, 0 },
     { "virtualbackground",	CKEYWORD, kwc_VirtualBackground },/*RFB VCOLOR*/
@@ -887,7 +988,7 @@ int do_single_keyword (keyword)
  	return 1;
 
       case kw0_SnapRealScreen:
-	Scr->SnapRealScreen = TRUE;
+	Scr->snapRealScreen = TRUE;
 	return 1;
 
       case kw0_NotVirtualGeometries:
@@ -915,6 +1016,12 @@ int do_single_keyword (keyword)
 	case kw0_DontInterpolateTitles: /* DSE */
 		Scr->DontInterpolateTitles = TRUE;
 		return 1;
+
+	/* djhjr - 1/6/98 */
+	case kw0_FixManagedVirtualGeometries:
+		Scr->FixManagedVirtualGeometries = TRUE;
+		return 1;
+
 	case kw0_FixTransientVirtualGeometries: /* DSE */
 		Scr->FixTransientVirtualGeometries = TRUE;
 		return 1;
@@ -922,6 +1029,40 @@ int do_single_keyword (keyword)
 		Scr->WarpSnug = TRUE;
 		return 1;
 
+	/* djhjr - 6/25/96 */
+	case kw0_ShallowReliefWindowButton:
+		Scr->ShallowReliefWindowButton = 1;
+		return 1;
+
+	/* djhjr - 4/18/96 */
+	case kw0_Use3DBorders:
+		Scr->use3Dborders = TRUE;
+		return 1;
+	case kw0_Use3DIconManagers:
+		Scr->use3Diconmanagers = TRUE;
+		return 1;
+	case kw0_Use3DMenus:
+		Scr->use3Dmenus = TRUE;
+		return 1;
+	case kw0_Use3DTitles:
+		Scr->use3Dtitles = TRUE;
+		return 1;
+
+	/* djhjr - 4/25/96 */
+	case kw0_SunkFocusWindowTitle:
+		Scr->SunkFocusWindowTitle = TRUE;
+		return 1;
+
+	/* djhjr - 9/21/96 */
+	case kw0_ButtonColorIsFrame:
+		Scr->ButtonColorIsFrame = TRUE;
+		return 1;
+
+	/* djhjr - 1/19/98 */
+	case kw0_BeNiceToColormap:
+		Scr->BeNiceToColormap = TRUE;
+		return 1;
+
     }
 
     return 0;
@@ -970,6 +1111,11 @@ int do_string_keyword (keyword, s)
 	if (!Scr->HaveFonts) Scr->MenuTitleFont.name = s;
 	return 1;
 
+	/* djhjr - 5/10/96 */
+      case kws_InfoFont:
+	if (!Scr->HaveFonts) Scr->InfoFont.name = s;
+	return 1;
+
       case kws_UnknownIcon:
 	if (Scr->FirstTime) GetUnknownIcon (s);
 	return 1;
@@ -1003,7 +1149,42 @@ int do_string_keyword (keyword, s)
       case kws_DoorFont:
 	if (!Scr->HaveFonts) Scr->DoorFont.name = s;
 	return 1;
-    }
+
+	/* djhjr - 5/15/96 */
+	case kws_ResizeRegion:
+		XmuCopyISOLatin1Lowered (s, s);
+		if (strcmp (s, "northwest") == 0)
+		{
+			Scr->ResizeX = R_NORTHWEST;
+			return 1;
+		}
+		else if (strcmp (s, "northeast") == 0)
+		{
+			Scr->ResizeX = R_NORTHEAST;
+			return 1;
+		}
+		if (strcmp (s, "southwest") == 0)
+		{
+			Scr->ResizeX = R_SOUTHWEST;
+			return 1;
+		}
+		else if (strcmp (s, "southeast") == 0)
+		{
+			Scr->ResizeX = R_SOUTHEAST;
+			return 1;
+		}
+		else if (strcmp (s, "centered") == 0)
+		{
+			Scr->ResizeX = R_CENTERED;
+			return 1;
+		}
+		else
+		{
+			twmrc_error_prefix();
+			fprintf (stderr, "Invalid ResizeRegion \"%s\"\n", s);
+			return 0;
+		}
+	}
 
     return 0;
 }
@@ -1063,13 +1244,21 @@ int do_number_keyword (keyword, num)
 	return 1;
 
       case kwn_PanDistanceX:
- 	if (Scr->FirstTime)
- 		Scr->VirtualDesktopPanDistanceX = (num * Scr->MyDisplayWidth) / 100;
- 	return 1;
+	if (Scr->FirstTime)
+	{
+		Scr->VirtualDesktopPanDistanceX = (num * Scr->MyDisplayWidth) / 100;
+		/* added this - djhjr - 1/4/98 */
+		if (Scr->VirtualDesktopPanDistanceX <= 0) Scr->VirtualDesktopPanDistanceX = 1;
+	}
+	return 1;
 
       case kwn_PanDistanceY:
- 	if (Scr->FirstTime)
- 		Scr->VirtualDesktopPanDistanceY = (num * Scr->MyDisplayHeight) / 100;
+	if (Scr->FirstTime)
+	{
+		Scr->VirtualDesktopPanDistanceY = (num * Scr->MyDisplayHeight) / 100;
+		/* added this - djhjr - 1/4/98 */
+		if (Scr->VirtualDesktopPanDistanceY <= 0) Scr->VirtualDesktopPanDistanceY = 1;
+	}
  	return 1;
 
 	case kwn_RaiseDelay: RaiseDelay = num; return 1;/*RAISEDELAY*/
@@ -1088,6 +1277,24 @@ int do_number_keyword (keyword, num)
 		Scr->AutoPanWarpWithRespectToRealScreen = (num<0)?0:(num>100)?100:num;
 		return 1;
 
+	/* djhjr - 4/18/96 */
+	case kwn_ThreeDBorderWidth:
+		if (Scr->FirstTime) Scr->ThreeDBorderWidth = num;
+		return 1;
+
+	/* djhjr - 4/19/96 */
+	case kwn_ClearShadowContrast:
+		if (Scr->FirstTime) Scr->ClearShadowContrast = num;
+		if (Scr->ClearShadowContrast <   0) Scr->ClearShadowContrast =   0;
+		if (Scr->ClearShadowContrast > 100) Scr->ClearShadowContrast = 100;
+		return 1;
+	case kwn_DarkShadowContrast:
+		if (Scr->FirstTime) Scr->DarkShadowContrast = num;
+		if (Scr->DarkShadowContrast <   0) Scr->DarkShadowContrast =   0;
+		if (Scr->DarkShadowContrast > 100) Scr->DarkShadowContrast = 100;
+		return 1;
+
+
     }
 
     return 0;
@@ -1225,7 +1432,7 @@ int do_color_keyword (keyword, colormode, s)
 /*
  * put_pixel_on_root() Save a pixel value in twm root window color property.
  */
-put_pixel_on_root(pixel)
+void put_pixel_on_root(pixel)
     Pixel pixel;
 {
   int           i, addPixel = 1;
@@ -1251,7 +1458,7 @@ put_pixel_on_root(pixel)
 /*
  * do_string_savecolor() save a color from a string in the twmrc file.
  */
-int do_string_savecolor(colormode, s)
+void do_string_savecolor(colormode, s)
      int colormode;
      char *s;
 {
@@ -1266,7 +1473,7 @@ int do_string_savecolor(colormode, s)
 typedef struct _cnode {int i; struct _cnode *next;} Cnode, *Cptr;
 Cptr chead = NULL;
 
-int do_var_savecolor(key)
+void do_var_savecolor(key)
 int key;
 {
   Cptr cptrav, cpnew;
@@ -1351,21 +1558,25 @@ static int ParseUsePPosition (s)
 }
 
 
-do_squeeze_entry (list, name, justify, num, denom)
+void do_squeeze_entry (list, name, justify, num, denom)
     name_list **list;			/* squeeze or dont-squeeze list */
     char *name;				/* window name */
     int justify;			/* left, center, right */
-    int num;				/* signed num */
-    int denom;				/* 0 or indicates fraction denom */
+    int num;				/* signed pixel count or fraction num */
+    int denom;				/* signed 0 or indicates fraction denom */
 {
     int absnum = (num < 0 ? -num : num);
+    int absdenom = (denom < 0 ? -denom : denom);
 
+    if (num < 0) {
+	twmrc_error_prefix();
+	fprintf (stderr, "SqueezeTitle numerator %d made positive\n", num);
+    }
     if (denom < 0) {
 	twmrc_error_prefix();
-	fprintf (stderr, "negative SqueezeTitle denominator %d\n", denom);
-	return;
+	fprintf (stderr, "SqueezeTitle denominator %d made positive\n", denom);
     }
-    if (absnum > denom && denom != 0) {
+    if (absnum > absdenom && denom != 0) {
 	twmrc_error_prefix();
 	fprintf (stderr, "SqueezeTitle fraction %d/%d outside window\n",
 		 num, denom);
@@ -1373,10 +1584,10 @@ do_squeeze_entry (list, name, justify, num, denom)
     }
     if (denom == 1) {
 	twmrc_error_prefix();
-	fprintf (stderr, "useless SqueezeTitle faction %d/%d, assuming 0/0\n",
+	fprintf (stderr, "useless SqueezeTitle fraction %d/%d, assuming 0/0\n",
 		 num, denom);
-	num = 0;
-	denom = 0;
+	absnum = 0;
+	absdenom = 0;
     }
 
     if (HasShape) {
@@ -1390,8 +1601,8 @@ do_squeeze_entry (list, name, justify, num, denom)
 	    return;
 	}
 	sinfo->justify = justify;
-	sinfo->num = num;
-	sinfo->denom = denom;
+	sinfo->num = absnum;
+	sinfo->denom = absdenom;
 	AddToList (list, name, (char *) sinfo);
     }
 }
